@@ -1,14 +1,14 @@
+# -*- coding: utf-8 -*-
+import operator
 import weakref
 import numpy as np
 from ..Qt import QtGui, QtCore
-from ..python2_3 import sortList
 from .. import functions as fn
 from .GraphicsObject import GraphicsObject
 from .GraphicsWidget import GraphicsWidget
 from ..widgets.SpinBox import SpinBox
 from ..pgcollections import OrderedDict
 from ..colormap import ColorMap
-from ..python2_3 import cmp
 
 
 __all__ = ['TickSliderItem', 'GradientEditorItem']
@@ -22,12 +22,18 @@ Gradients = OrderedDict([
     ('cyclic', {'ticks': [(0.0, (255, 0, 4, 255)), (1.0, (255, 0, 0, 255))], 'mode': 'hsv'}),
     ('greyclip', {'ticks': [(0.0, (0, 0, 0, 255)), (0.99, (255, 255, 255, 255)), (1.0, (255, 0, 0, 255))], 'mode': 'rgb'}),
     ('grey', {'ticks': [(0.0, (0, 0, 0, 255)), (1.0, (255, 255, 255, 255))], 'mode': 'rgb'}),
+    # Perceptually uniform sequential colormaps from Matplotlib 2.0
+    ('viridis', {'ticks': [(0.0, (68, 1, 84, 255)), (0.25, (58, 82, 139, 255)), (0.5, (32, 144, 140, 255)), (0.75, (94, 201, 97, 255)), (1.0, (253, 231, 36, 255))], 'mode': 'rgb'}),
+    ('inferno', {'ticks': [(0.0, (0, 0, 3, 255)), (0.25, (87, 15, 109, 255)), (0.5, (187, 55, 84, 255)), (0.75, (249, 142, 8, 255)), (1.0, (252, 254, 164, 255))], 'mode': 'rgb'}),
+    ('plasma', {'ticks': [(0.0, (12, 7, 134, 255)), (0.25, (126, 3, 167, 255)), (0.5, (203, 71, 119, 255)), (0.75, (248, 149, 64, 255)), (1.0, (239, 248, 33, 255))], 'mode': 'rgb'}),
+    ('magma', {'ticks': [(0.0, (0, 0, 3, 255)), (0.25, (80, 18, 123, 255)), (0.5, (182, 54, 121, 255)), (0.75, (251, 136, 97, 255)), (1.0, (251, 252, 191, 255))], 'mode': 'rgb'}),
 ])
 
 def addGradientListToDocstring():
     """Decorator to add list of current pre-defined gradients to the end of a function docstring."""
     def dec(fn):
-        fn.__doc__ = fn.__doc__ + str(Gradients.keys()).strip('[').strip(']')
+        if fn.__doc__ is not None:
+            fn.__doc__ = fn.__doc__ + str(Gradients.keys()).strip('[').strip(']')
         return fn
     return dec
 
@@ -346,8 +352,7 @@ class TickSliderItem(GraphicsWidget):
     def listTicks(self):
         """Return a sorted list of all the Tick objects on the slider."""
         ## public
-        ticks = list(self.ticks.items())
-        sortList(ticks, lambda a,b: cmp(a[1], b[1]))  ## see pyqtgraph.python2_3.sortList
+        ticks = sorted(self.ticks.items(), key=operator.itemgetter(1))
         return ticks
 
 
@@ -433,8 +438,14 @@ class GradientEditorItem(TickSliderItem):
             label = QtGui.QLabel()
             label.setPixmap(px)
             label.setContentsMargins(1, 1, 1, 1)
+            labelName = QtGui.QLabel(g)
+            hbox = QtGui.QHBoxLayout()
+            hbox.addWidget(labelName)
+            hbox.addWidget(label)
+            widget = QtGui.QWidget()
+            widget.setLayout(hbox)
             act = QtGui.QWidgetAction(self)
-            act.setDefaultWidget(label)
+            act.setDefaultWidget(widget)
             act.triggered.connect(self.contextMenuClicked)
             act.name = g
             self.menu.addAction(act)
@@ -450,7 +461,20 @@ class GradientEditorItem(TickSliderItem):
         self.addTick(1, QtGui.QColor(255,0,0), True)
         self.setColorMode('rgb')
         self.updateGradient()
-    
+        self.linkedGradients = {}
+
+    def showTicks(self, show=True):
+        for tick in self.ticks.keys():
+            if show:
+                tick.show()
+                orig = getattr(self, '_allowAdd_backup', None)
+                if orig: 
+                    self.allowAdd = orig
+            else:
+                self._allowAdd_backup = self.allowAdd
+                self.allowAdd = False #block tick creation
+                tick.hide()
+
     def setOrientation(self, orientation):
         ## public
         """
@@ -649,7 +673,7 @@ class GradientEditorItem(TickSliderItem):
             s = s1 * (1.-f) + s2 * f
             v = v1 * (1.-f) + v2 * f
             c = QtGui.QColor()
-            c.setHsv(h,s,v)
+            c.setHsv(*map(int, [h,s,v]))
             if toQColor:
                 return c
             else:
@@ -753,7 +777,9 @@ class GradientEditorItem(TickSliderItem):
         for t in self.ticks:
             c = t.color
             ticks.append((self.ticks[t], (c.red(), c.green(), c.blue(), c.alpha())))
-        state = {'mode': self.colorMode, 'ticks': ticks}
+        state = {'mode': self.colorMode, 
+                 'ticks': ticks,
+                 'ticksVisible': next(iter(self.ticks)).isVisible()}
         return state
         
     def restoreState(self, state):
@@ -778,6 +804,8 @@ class GradientEditorItem(TickSliderItem):
         for t in state['ticks']:
             c = QtGui.QColor(*t[1])
             self.addTick(t[0], c, finish=False)
+        self.showTicks( state.get('ticksVisible', 
+                                  next(iter(self.ticks)).isVisible()) )
         self.updateGradient()
         self.sigGradientChangeFinished.emit(self)
         
@@ -793,6 +821,18 @@ class GradientEditorItem(TickSliderItem):
         self.updateGradient()
         self.sigGradientChangeFinished.emit(self)
 
+    def linkGradient(self, slaveGradient, connect=True):
+        if connect:
+            fn = lambda g, slave=slaveGradient:slave.restoreState(
+                                                     g.saveState())
+            self.linkedGradients[id(slaveGradient)] = fn
+            self.sigGradientChanged.connect(fn)
+            self.sigGradientChanged.emit(self)
+        else:
+            fn = self.linkedGradients.get(id(slaveGradient), None)
+            if fn:
+                self.sigGradientChanged.disconnect(fn)
+
 
 class Tick(QtGui.QGraphicsWidget):  ## NOTE: Making this a subclass of GraphicsObject instead results in
                                     ## activating this bug: https://bugreports.qt-project.org/browse/PYSIDE-86
@@ -863,8 +903,8 @@ class Tick(QtGui.QGraphicsWidget):  ## NOTE: Making this a subclass of GraphicsO
                 self.view().tickMoveFinished(self)
 
     def mouseClickEvent(self, ev):
-        if  ev.button() == QtCore.Qt.RightButton and self.moving:
-            ev.accept()
+        ev.accept()
+        if ev.button() == QtCore.Qt.RightButton and self.moving:
             self.setPos(self.startPosition)
             self.view().tickMoved(self, self.startPosition)
             self.moving = False
@@ -872,7 +912,6 @@ class Tick(QtGui.QGraphicsWidget):  ## NOTE: Making this a subclass of GraphicsO
             self.sigMoved.emit(self)
         else:
             self.view().tickClicked(self, ev)
-            ##remove
 
     def hoverEvent(self, ev):
         if (not ev.isExit()) and ev.acceptDrags(QtCore.Qt.LeftButton):
@@ -938,4 +977,3 @@ class TickMenu(QtGui.QMenu):
     #    self.fracPosSpin.blockSignals(True)
     #    self.fracPosSpin.setValue(self.sliderItem().tickValue(self.tick()))
     #    self.fracPosSpin.blockSignals(False)
-
