@@ -1,4 +1,4 @@
-from ..Qt import QtCore, QtGui, QtOpenGL, USE_PYQT5
+from ..Qt import QtCore, QtGui, QtOpenGL, QT_LIB
 from OpenGL.GL import *
 import OpenGL.GL.framebufferobjects as glfbo
 import numpy as np
@@ -16,9 +16,13 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         - Axis/grid display
         - Export options
 
+
+    High-DPI displays: Qt5 should automatically detect the correct resolution.
+    For Qt4, specify the ``devicePixelRatio`` argument when initializing the
+    widget (usually this value is 1-2).
     """
     
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, devicePixelRatio=None):
         global ShareWidget
 
         if ShareWidget is None:
@@ -37,6 +41,7 @@ class GLViewWidget(QtOpenGL.QGLWidget):
             'azimuth': 45,            ## camera's azimuthal angle in degrees 
                                       ## (rotation around z-axis 0 points along x-axis)
             'viewport': None,         ## glViewport params; None == whole widget
+            'devicePixelRatio': devicePixelRatio,
         }
         self.setBackgroundColor('k')
         self.items = []
@@ -79,10 +84,21 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         
     def getViewport(self):
         vp = self.opts['viewport']
+        dpr = self.devicePixelRatio()
         if vp is None:
-            return (0, 0, self.width(), self.height())
+            return (0, 0, int(self.width() * dpr), int(self.height() * dpr))
         else:
-            return vp
+            return tuple([int(x * dpr) for x in vp])
+        
+    def devicePixelRatio(self):
+        dpr = self.opts['devicePixelRatio']
+        if dpr is not None:
+            return dpr
+        
+        if hasattr(QtOpenGL.QGLWidget, 'devicePixelRatio'):
+            return QtOpenGL.QGLWidget.devicePixelRatio(self)
+        else:
+            return 1.0
         
     def resizeGL(self, w, h):
         pass
@@ -97,9 +113,9 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         glMultMatrixf(a.transpose())
 
     def projectionMatrix(self, region=None):
-        # Xw = (Xnd + 1) * width/2 + X
         if region is None:
-            region = (0, 0, self.width(), self.height())
+            dpr = self.devicePixelRatio()
+            region = (0, 0, self.width() * dpr, self.height() * dpr)
         
         x0, y0, w, h = self.getViewport()
         dist = self.opts['distance']
@@ -110,8 +126,6 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         r = nearClip * np.tan(fov * 0.5 * np.pi / 180.)
         t = r * h / w
 
-        # convert screen coordinates (region) to normalized device coordinates
-        # Xnd = (Xw - X0) * 2/width - 1
         ## Note that X0 and width in these equations must be the values used in viewport
         left  = r * ((region[0]-x0) * (2.0/w) - 1)
         right = r * ((region[0]+region[2]-x0) * (2.0/w) - 1)
@@ -222,6 +236,8 @@ class GLViewWidget(QtOpenGL.QGLWidget):
                     glPopMatrix()
             
     def setCameraPosition(self, pos=None, distance=None, elevation=None, azimuth=None):
+        if pos is not None:
+            self.opts['center'] = pos
         if distance is not None:
             self.opts['distance'] = distance
         if elevation is not None:
@@ -230,8 +246,6 @@ class GLViewWidget(QtOpenGL.QGLWidget):
             self.opts['azimuth'] = azimuth
         self.update()
         
-        
-        
     def cameraPosition(self):
         """Return current position of camera based on center, dist, elevation, and azimuth"""
         center = self.opts['center']
@@ -250,24 +264,41 @@ class GLViewWidget(QtOpenGL.QGLWidget):
     def orbit(self, azim, elev):
         """Orbits the camera around the center position. *azim* and *elev* are given in degrees."""
         self.opts['azimuth'] += azim
-        #self.opts['elevation'] += elev
         self.opts['elevation'] = np.clip(self.opts['elevation'] + elev, -90, 90)
         self.update()
         
-    def pan(self, dx, dy, dz, relative=False):
+    def pan(self, dx, dy, dz, relative='global'):
         """
         Moves the center (look-at) position while holding the camera in place. 
         
-        If relative=True, then the coordinates are interpreted such that x
-        if in the global xy plane and points to the right side of the view, y is
-        in the global xy plane and orthogonal to x, and z points in the global z
-        direction. Distances are scaled roughly such that a value of 1.0 moves
+        ==============  =======================================================
+        **Arguments:**
+        *dx*            Distance to pan in x direction
+        *dy*            Distance to pan in y direction
+        *dx*            Distance to pan in z direction
+        *relative*      String that determines the direction of dx,dy,dz. 
+                        If "global", then the global coordinate system is used.
+                        If "view", then the z axis is aligned with the view
+                        direction, and x and y axes are inthe plane of the
+                        view: +x points right, +y points up. 
+                        If "view-upright", then x is in the global xy plane and
+                        points to the right side of the view, y is in the
+                        global xy plane and orthogonal to x, and z points in
+                        the global z direction.
+        ==============  =======================================================
+        
+        Distances are scaled roughly such that a value of 1.0 moves
         by one pixel on screen.
         
+        Prior to version 0.11, *relative* was expected to be either True (x-aligned) or
+        False (global). These values are deprecated but still recognized.
         """
-        if not relative:
+        # for backward compatibility:
+        relative = {True: "view-upright", False: "global"}.get(relative, relative)
+        
+        if relative == 'global':
             self.opts['center'] += QtGui.QVector3D(dx, dy, dz)
-        else:
+        elif relative == 'view-upright':
             cPos = self.cameraPosition()
             cVec = self.opts['center'] - cPos
             dist = cVec.length()  ## distance from camera to center
@@ -277,6 +308,21 @@ class GLViewWidget(QtOpenGL.QGLWidget):
             xVec = QtGui.QVector3D.crossProduct(zVec, cVec).normalized()
             yVec = QtGui.QVector3D.crossProduct(xVec, zVec).normalized()
             self.opts['center'] = self.opts['center'] + xVec * xScale * dx + yVec * xScale * dy + zVec * xScale * dz
+        elif relative == 'view':
+            # pan in plane of camera
+            elev = np.radians(self.opts['elevation'])
+            azim = np.radians(self.opts['azimuth'])
+            fov = np.radians(self.opts['fov'])
+            dist = (self.opts['center'] - self.cameraPosition()).length()
+            fov_factor = np.tan(fov / 2) * 2
+            scale_factor = dist * fov_factor / self.width()
+            z = scale_factor * np.cos(elev) * dy
+            x = scale_factor * (np.sin(azim) * dx - np.sin(elev) * np.cos(azim) * dy)
+            y = scale_factor * (np.cos(azim) * dx + np.sin(elev) * np.sin(azim) * dy)
+            self.opts['center'] += QtGui.QVector3D(x, -y, z)
+        else:
+            raise ValueError("relative argument must be global, view, or view-upright")
+        
         self.update()
         
     def pixelSize(self, pos):
@@ -301,13 +347,15 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         self.mousePos = ev.pos()
         
         if ev.buttons() == QtCore.Qt.LeftButton:
-            self.orbit(-diff.x(), diff.y())
-            #print self.opts['azimuth'], self.opts['elevation']
+            if (ev.modifiers() & QtCore.Qt.ControlModifier):
+                self.pan(diff.x(), diff.y(), 0, relative='view')
+            else:
+                self.orbit(-diff.x(), diff.y())
         elif ev.buttons() == QtCore.Qt.MidButton:
             if (ev.modifiers() & QtCore.Qt.ControlModifier):
-                self.pan(diff.x(), 0, diff.y(), relative=True)
+                self.pan(diff.x(), 0, diff.y(), relative='view-upright')
             else:
-                self.pan(diff.x(), diff.y(), 0, relative=True)
+                self.pan(diff.x(), diff.y(), 0, relative='view-upright')
         
     def mouseReleaseEvent(self, ev):
         pass
@@ -325,7 +373,7 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         
     def wheelEvent(self, ev):
         delta = 0
-        if not USE_PYQT5:
+        if QT_LIB in ['PyQt4', 'PySide']:
             delta = ev.delta()
         else:
             delta = ev.angleDelta().x()
@@ -379,15 +427,13 @@ class GLViewWidget(QtOpenGL.QGLWidget):
     def checkOpenGLVersion(self, msg):
         ## Only to be called from within exception handler.
         ver = glGetString(GL_VERSION).split()[0]
-        if int(ver.split('.')[0]) < 2:
+        if int(ver.split(b'.')[0]) < 2:
             from .. import debug
-            pyqtgraph.debug.printExc()
+            debug.printExc()
             raise Exception(msg + " The original exception is printed above; however, pyqtgraph requires OpenGL version 2.0 or greater for many of its 3D features and your OpenGL version is %s. Installing updated display drivers may resolve this issue." % ver)
         else:
             raise
             
-
-            
     def readQImage(self):
         """
         Read the current buffer pixels out as a QImage.
@@ -411,7 +457,6 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         img = fn.makeQImage(pixels, transpose=False)
         return img
         
-        
     def renderToArray(self, size, format=GL_BGRA, type=GL_UNSIGNED_BYTE, textureSize=1024, padding=256):
         w,h = map(int, size)
         
@@ -450,6 +495,7 @@ class GLViewWidget(QtOpenGL.QGLWidget):
                     glfbo.glFramebufferTexture2D(glfbo.GL_FRAMEBUFFER, glfbo.GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, tex, 0)
                     
                     self.paintGL(region=(x, h-y-h2, w2, h2), viewport=(0, 0, w2, h2))  # only render sub-region
+                    glBindTexture(GL_TEXTURE_2D, tex) # fixes issue #366
                     
                     ## read texture back to array
                     data = glGetTexImage(GL_TEXTURE_2D, 0, format, type)
